#define DEVBOX_DELL				0
#define DEVBOX_HP				1
#define VORTEX_DEVNODE_TYPE		DEVBOX_HP

#include <vortex/acpi/acpiheaders.h>
#include <vortex/acpi/platform/acpi_vortex.h>
#include <vortex/acpi/platform/vortex_acpi_pstates.h>
#include <vortex/acpi/pmcoremon.h>
#include <vortex/acpi/gphr.h>
#include <vortex/acpi/pht.h>
#include <vortex/acpi/gphtpredictor.h>
#include <vortex/cpumux.h>
#include <vortex/acpi/load_balancing_scheduler.h>
//FOR DEBUGGING
#include <vortex/klib/priqueue.h>


/******************************************************************************
 *
 * FUNCTION:     AcpiOsInitialize
 *
 * PARAMETERS:  None
 *
 * RETURN:      AE_OK		Initialization successfull.
 *				Other errors can be added. The actual error codes do not matter
 *				for anything but debugging.
 *
 * DESCRIPTION: This function allows the OSL to initialize itself. Is is called 
 *				during initialization of the ACPICA subsystem.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsInitialize (void)
{
	/*Initialize scheduler thang*/
	if (init_acpi_scheduler() != AE_OK) {
		syslog(LOG_CRIT, "Initialization of acpi scheduler failed...\n");
		return AE_ERROR;
	}
	
	if (init_interrupt_storage() != AE_OK) {
		syslog(LOG_CRIT, "Initialization of acpi interrupt storage failed...\n");
		return AE_ERROR;
	}

	return AE_OK;
}


static void core_state_monitoring_thread(uint32 cpuid)
{
	uint64  timestamps[1000];
    uint64  current_time;
    int32   core_ishalted[1000];
    int32   i, output_offset;
    objref_t outputfile_obj;
    file_t   *outputfile;
    membuf_t *output_mb = (membuf_t*) NULL;
    vxerr_t   vxerr;

    syslog(LOG_DEBUG, "about to run monitoring thread for core %d on core %d", cpuid, CPU_ID);

    i = 1;
    core_ishalted[0] = 0;
    timestamps[0] = arch_timestamp_usec();
    while (i < 1000) {
        
        while (cpulocal[cpuid].cl_current_power_state == core_ishalted[i-1]) {
            current_time = arch_timestamp_usec();
        }

        core_ishalted[i] = cpulocal[cpuid].cl_current_power_state;
        timestamps[i] = current_time;
        i++;
    }

    syslog(LOG_DEBUG, "Done monitoring core %d - about to dump to file", cpuid);

    for (i = 1; i < 1000; i++)
        syslog(LOG_DEBUG, "Timestamp number %d : %D Corestate: %d, time since last statechange is %d", i, timestamps[i], core_ishalted[i], timestamps[i] - timestamps[i-1]);

    syslog(LOG_CRIT, "Verify output!");
}

/*Function for testing stuff wrt. ACPI functionality.*/

vxerr_t init151_acpicaTest(void)
{
	ACPI_STATUS		status;
	pstate_array_t	*pstate_array0, *pstate_array1, *pstate_array2, *pstate_array3, *pstate_array4, *pstate_array5, *pstate_array6, *pstate_array7;
	cstate_array_t	*cstate_array0, *cstate_array1, *cstate_array2, *cstate_array3, *cstate_array4, *cstate_array5, *cstate_array6, *cstate_array7;
	uint32			use_ffh0, use_ffh1, use_ffh2, use_ffh3, use_ffh4, use_ffh5, use_ffh6, use_ffh7;
	uint64			control_register0, status_register0, control_register1, status_register1, control_register2, status_register2, control_register3, status_register3, control_register4, status_register4, control_register5, status_register5, control_register6, status_register6, control_register7, status_register7;
	vxerr_t			vxerr;
	objref_t		tcbref, timerref;
	tcb_t			*tcb;

	ExecuteOSI();
	//display_all_devices();
	if (VORTEX_DEVNODE_TYPE == DEVBOX_DELL) {
		/*The following functions are not supported by the HP voxo-blade boxes*/
		ExecutePciHostBridge_OSC(_OSC_DWORD1_QUERY, _OSC_PCIHB_SUPPORT_FIELD, 0x00000000); 
		ExecutePciHostBridge_OSC(0x00000000, 0x00000000, 0x0000001F);
	}

	ExecuteCPUOSCQueries();
	enable_enhanced_speedstep();

	pstate_array0 = acpi_get_performance_supported_states(0);
	acpi_retrieve_performance_control_registers(0, &use_ffh0, &control_register0, &status_register0);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh0, control_register0, status_register0);

	pstate_array1 = acpi_get_performance_supported_states(1);
	acpi_retrieve_performance_control_registers(1, &use_ffh1, &control_register1, &status_register1);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh1, control_register1, status_register1);

	pstate_array2 = acpi_get_performance_supported_states(2);
	acpi_retrieve_performance_control_registers(2, &use_ffh2, &control_register2, &status_register2);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh2, control_register2, status_register2);

	pstate_array3 = acpi_get_performance_supported_states(3);
	acpi_retrieve_performance_control_registers(3, &use_ffh3, &control_register3, &status_register3);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh3, control_register3, status_register3);

	pstate_array4 = acpi_get_performance_supported_states(4);
	acpi_retrieve_performance_control_registers(4, &use_ffh4, &control_register4, &status_register4);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh4, control_register4, status_register4);

	pstate_array5 = acpi_get_performance_supported_states(5);
	acpi_retrieve_performance_control_registers(5, &use_ffh5, &control_register5, &status_register5);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh5, control_register5, status_register5);

	pstate_array6 = acpi_get_performance_supported_states(6);
	acpi_retrieve_performance_control_registers(6, &use_ffh6, &control_register6, &status_register6);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh6, control_register6, status_register6);

	pstate_array7 = acpi_get_performance_supported_states(7);
	acpi_retrieve_performance_control_registers(7, &use_ffh7, &control_register7, &status_register7);
	syslog(LOG_DEBUG, "Intel Enhances Speedstep? %d, control_register: %x, status_register %x", use_ffh7, control_register7, status_register7);
    
	lookAtPSD(0);

	lookAtCST(0);

    init_pm_coremon();

	syslog(LOG_DEBUG, "Passed acpica test function");	
}

/*Initializing the acpica subsystem using a Vortex inittab function*/
vxerr_t init150_acpicaFull(void)
{
	ACPI_STATUS             Status;

	syslog(LOG_DEBUG, "#############################################");
	syslog(LOG_DEBUG, "######## STARTING ACPI INIT SEQUENCE ########");
	syslog(LOG_DEBUG, "#############################################");
    /* Initialize the ACPICA subsystem */
    Status = AcpiInitializeSubsystem ();
    if (ACPI_FAILURE (Status)) {
		syslog(LOG_CRIT, "Error while initializing ACPICA");
		return VXERR_INTERNAL;
    }
	syslog(LOG_DEBUG, "Acpi Subsystem initialized");

    /* Initialize the ACPICA Table Manager and get all ACPI tables */
    Status = AcpiInitializeTables (NULL, 16, FALSE);
    if (ACPI_FAILURE (Status)) {
		syslog(LOG_CRIT, "Error while initializing ACPICA Table Manager");
        return VXERR_INTERNAL;
    }
	syslog(LOG_DEBUG, "Acpi Tables initialized");

    /* Create the ACPI namespace from ACPI tables */
    Status = AcpiLoadTables ();
    if (ACPI_FAILURE (Status)) {
		syslog(LOG_CRIT, "Error while loading ACPI tables");
        return VXERR_INTERNAL;
    }
	syslog(LOG_DEBUG, "Acpi Tables loaded");

    /* Initialize the ACPI hardware */
    Status = AcpiEnableSubsystem (ACPI_FULL_INITIALIZATION);
    if (ACPI_FAILURE (Status)) {
		syslog(LOG_CRIT, "Error while enabling ACPICA");
        return VXERR_INTERNAL;
    }
	syslog(LOG_DEBUG, "Acpi Subsystem enabled");

    /* Complete the ACPI namespace object initialization */
    Status = AcpiInitializeObjects (ACPI_FULL_INITIALIZATION);
    if (ACPI_FAILURE (Status)) {
		syslog(LOG_CRIT, "Error while initializing ACPICA Objects");
        return VXERR_INTERNAL;
    }
	syslog(LOG_DEBUG, "Acpi Objects initialized");

	
	syslog(LOG_DEBUG, "#############################################");
	syslog(LOG_DEBUG, "######## FINISHED ACPI INIT SEQUENCE ########");
	syslog(LOG_DEBUG, "#############################################");
	return VXERR_OK;
}

/******************************************************************************
 *
 * FUNCTION:     AcpiOsTerminate
 *
 * PARAMETERS:  None
 *
 * RETURN:      AE_OK		Termination successful.
 *				Other erros can be added. The actual error codes do not matter 
 *				for anything but debugging.
 *
 * DESCRIPTION: This function allows the OSL to cleanup and terminate. Is is 
 *				called during the termination of the ACPICA subsystem.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsTerminate (void)
{
	/*Everything is allocated statically, nothing to do here*/
	return AE_OK;
}


/******************************************************************************
 *
 * FUNCTION:     AcpiOsGetRootPointer
 *
 * PARAMETERS:  None
 *
 * RETURN:      Address	The physical address to the RSDP
 *
 * DESCRIPTION: This function returns the physical address of the  ACPI RSDP 
 *				(Root System Description Pointer) table. The mechanism used to 
 *				obtain this pointer is platform and/or OS dependent. There are 
 *				two primary methods used to obtain this pointer and thus 
 *				implement the interface:
 *
 *				1) On IA-32 platforms, the RSDP is obtained by searching the 
 *				   first megabyte og physical memory for the RSDP signature 
 *				   ("RSD PTR"). On these platforms, this interface should be 
 *				   implemented via a call to the AcpiFindRootPointer interface.
 *
 *				2) On IA-64 platforms, the RSDP is obtained from the EFI 
 *				   (Extended Firmware Interface). The pointer in the EFI 
 *				   information block that is passed to the OS at OS startup.
 *
 *****************************************************************************/
ACPI_PHYSICAL_ADDRESS AcpiOsGetRootPointer (void)
{
#ifdef ACPI_IA_32
	/*If IA-32*/
	ACPI_PHYSICAL_ADDRESS pa = 0;
	AcpiFindRootPointer(&pa); 
	return pa;

#endif

#ifdef ACPIA_IA_64
	/*If IA-64 (Intel Intanium*/
	return (ACPI_PHYSICAL_ADDRESS)NULL;

#endif
}


/*
 * Directory manipulation
 */
void *AcpiOsOpenDirectory (char *Pathname, char *WildcardSpec, char RequestedFileType)
{
	syslog(LOG_CRIT, "AcpiOsOpenDirectory - not implemented\n");
}

/* RequesteFileType values */

char *AcpiOsGetNextFilename (void *DirHandle)
{
	syslog(LOG_CRIT, "AcpiOsGetNextFilename - not implemented\n");
}

void AcpiOsCloseDirectory (void *DirHandle)
{
	syslog(LOG_CRIT, "AcpiOsCloseDirectory - not implemented\n");
}



/* The following table functions are used only by the AcpiDump utility - Should be
 * innecessary*/

/******************************************************************************
 *
 * FUNCTION:	AcpiOsGetTableByName
 *
 * PARAMETERS:  Signature	- The uppercase ACPI signature for the requested 
 *							  table. This must be a 4 character null-terminated
 *							  ASCII string.
 *				Instance	- Used to obtain tables that are allowed to have 
 *							  multiple instances (SSDT or UEFI). For regular 
 *							  ACPI tables, this parameter is ignored.
 *				OutTable	- A pointer location where the table is to be 
 *							  returned.
 *				OutAddress  - A pointer to location where the physical address 
 *							  of the table is returned.
 *
 * RETURN:      AE_OK				The table was successfully loaded and 
 *									returned.
 *				AE_BAD_PARAMETER	The OutTable or OutAddress pointer is NULL.
 *				AE_LIMIT			The last valid Instance value has been
 *									reached. There is no table corresponding 
 *									to the Instance.
 *				AE_NO_ACPI_TABLES	The ACPI tables could not be located.
 *				AE_NO_MEMORY		Insufficient dynamic memory to create a 
 *									buffer for the ACPI table.
 *				AE_NOT_FOUND		An ACPI table with Signature was not 
 *									found.
 *				AE_SUPPORT			The function is not currently supported 
 *									at the host.
 *
 * DESCRIPTION: This function obtains an installed ACPI table by specifying a 
 *				table signature. Tables that can have multiple instances are 
 *				also supported via the Instance parameter. Valid instance 
 *				values are 0 thoufh n-1, where n is the number of currently 
 *				installed SSDTs.
 *
 *				If the operation fails, and appropriate status will be 
 *				returned and the contents of OutTable and OutAddress are 
 *				undefined.
 *
 *				The OutAddress parameter will be set to zero if the physical
 *				address is not available.
 *
 *				AcpiOsGetTableByIndex allocates a buffer for the ACPI table 
 *				that should be freed by the caller when finished with it.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsGetTableByName (char *Signature, UINT32 Instance, ACPI_TABLE_HEADER **Table, ACPI_PHYSICAL_ADDRESS *Address)
{
	syslog(LOG_CRIT, "AcpiOsGetTableByName - not implemented\n");
}


/******************************************************************************
 *
 * FUNCTION:	AcpiOsGetTableByIndex
 *
 * PARAMETERS:  TableIndex	- Index of the requested table. The index does not
 *							  necessarily correspond to the ordering of the 
 *							  RSDT/XSDT.
 *				OutTable	- A pointer location where the table is to be 
 *							  returned.
 *				OutAddress  - A pointer to location where the physical address 
 *							  of the table is returned.
 *
 * RETURN:      AE_OK				The table was successfully loaded and 
 *									returned.
 *				AE_BAD_PARAMETER	The OutTable or OutAddress pointer is NULL.
 *				AE_LIMIT			The last valid TableIndex value has been
 *									reached. There is no table corresponding 
 *									to TableIndex.
 *				AE_NO_ACPI_TABLES	The ACPI tables could not be located.
 *				AE_NO_MEMORY		Insufficient dynamic memory to create a 
 *									buffer for the ACPI table.
 *				AE_SUPPORT			The function is not currently supported 
 *									at the host.
 *
 * DESCRIPTION: This function obtains an installed ACPI table by specifying a 
 *				table index value. Valid table index values are 0 through n-1, 
 *				where n is the number of currently installed ACPI tables. This
 *				function is usefull for iterating through the entire set of 
 *				installed ACPI tables. To obtain a specific ACPI table, use 
 *				AcpiOsGetTableByName or AcpiOsGetTableByAddress.
 *
 *				If the operation fails, and appropriate status will be 
 *				returned and the contents of OutTable and OutAddress are 
 *				undefined.
 *
 *				AcpiOsGetTableByIndex allocates a buffer for the ACPI table 
 *				that should be freed by the caller when finished with it.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsGetTableByIndex (UINT32 Index, ACPI_TABLE_HEADER **Table, UINT32 *Instance, ACPI_PHYSICAL_ADDRESS *Address)
{
	syslog(LOG_CRIT, "AcpiOsGetTableByIndex - not implemented\n");
}


/******************************************************************************
 *
 * FUNCTION:	AcpiOsGetTableByAddress
 *
 * PARAMETERS:  Address		- Memory physical address of the requested ACPI 
 *							  table.
 *				OutTable	- A pointer location where the table is to be 
 *							  returned.
 *
 * RETURN:      AE_OK				The table was successfully loaded and 
 *									returned.
 *				AE_BAD_PARAMETER	The OutTable pointer is NULL.
 *				AE_NO_ACPI_TABLES	The ACPI tables could not be located.
 *				AE_NO_MEMORY		Insufficient dynamic memory to create a 
 *									buffer for the ACPI table.
 *				AE_NOT_FOUND		A valid ACPI table was not found at the 
 *									specified address.
 *				AE_SUPPORT			The function is not currently supported 
 *									at the host.
 *
 * DESCRIPTION: This function obtains an ACPI table by specifying a physical 
 *				address. It is most useful for getting a table that is 
 *				dynamically loaded and is not actually present in the 
 *				RSDT/XSDT. NOTE: It may not be possible to support this 
 *				function on all hosts. 
 *
 *				If the operation fails, an appropriate status will be returned
 *				and the contents of the OutTable is undefined.
 *
 *				AcpiOsGetTableByAddress allocates a buffer for the ACPI table 
 *				that should be freed by the caller when finished with it.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsGetTableByAddress (ACPI_PHYSICAL_ADDRESS Address, ACPI_TABLE_HEADER **Table)
{
	syslog(LOG_CRIT, "AcpiOsGetTableByAddress - not implemented\n");
}

