#include <vortex/acpi/acpiheaders.h>


/*
 * Debug input
 */

/******************************************************************************
 *
 * FUNCTION:     AcpiOsGetLine
 *
 * PARAMETERS:  Buffer				- Where to return the input line.
 *				BufferLength		- Length of the buffer (max data to return).
 *				*BytesRead			- Where the actual byte count is returned.
 *
 * RETURN:      AE_OK				The line was successfully obtained.
 *				AE_BUFFER_OVERFLOW	The line was to large for the input buffer.
 *
 * DESCRIPTION: Get a line of input from the debugger command line. The purpose 
 *				of this function is to support the ACPI debugger, and it is 
 *				therefore optional depending on whether ACPI debugger support 
 *				is desired.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsGetLine (char *Buffer, UINT32 BufferLength, UINT32 *BytesRead)
{
	return AE_OK;
}

/******************************************************************************
 *
 * FUNCTION:     AcpiOsSignal
 *
 * PARAMETERS:  Function			- Signal to be sent to the host operating 
 *									  system - one of these manifest constants:
 *										ACPI_SIGNAL_FATAL
 *										ACPI_SIGNAL_BREAKPOINT
 *				Info				- No description given...?
 *
 * RETURN:      AE_OK				The line was successfully obtained.
 *				AE_BUFFER_OVERFLOW	The line was to large for the input buffer.
 *
 * DESCRIPTION: This function is used to pass various signals and notifications 
 *				to the host operating system. The following signals are 
 *				supported:
 *
 *				ACPI_SIGNAL_FATAL:	The signal corresponds to the AML Fatal 
 *									opcode. It is sent to the host OS only when
 *									this opcode is encountered in the AML 
 *									stream. The host OS may or may not return
 *									control from this signal. The definition of
 *									the Info structure for this signal is as 
 *									follows:
 *									typedef struct AcpiFatalInfo
 *									{
 *										UINT32	Type;
 *										UINT32	Code;
 *										UINT32	Argument;
 *									} ACPI_SIGNAL_FATAL_INFO;
 *
 *				ACPI_SIGNAL_BREAKPOINT:	The signal corresponds to the AML 
 *										Breakpoint opcode. The OSL implements a 
 *										"Breakpoint" operation as appropriate for 
 *										the host OS. If in debug mode, this 
 *										interface may cause a break into the 
 *										host kernel debugger. The definition of 
 *										the Info structure for this signal is as 
 *										follows:
 *										char *BreakpointMessage;
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsSignal (UINT32 Function, void *Info)
{
	ACPI_SIGNAL_FATAL_INFO      *fatal;

	switch ((uint32)Function) 
	{
		case ACPI_SIGNAL_FATAL:
			fatal = (ACPI_SIGNAL_FATAL_INFO *)Info;
			syslog(LOG_DEBUG, "ACPI fatal signal, type 0x%x code 0x%x argument 0x%x", fatal->Type, fatal->Code, fatal->Argument);
			break;
			
		case ACPI_SIGNAL_BREAKPOINT:
			syslog(LOG_DEBUG, "ACPI Breakpoint signal, message: %s \n", Info);
			break;

		default:
			return AE_BAD_PARAMETER;
	}
	
	return AE_OK;
}