/*
#include <vortex/common.h>
#include <vortex/acpi/platform/acenv.h>
#include <vortex/acpi/actypes.h>
#include <vortex/acpi/actbl.h>
#include <vortex/acpi/acpiosxf.h>
#include <vortex/klib/print.h>
*/
#include <vortex/acpi/acpiheaders.h>

/*
 * Debug print routines
 */

/******************************************************************************
 *
 * FUNCTION:     AcpiOsPrintf
 *
 * PARAMETERS:  Format		- A standard print format string.
 *				...			- Variable printf parameter list.
 *
 * RETURN:      None
 *
 * DESCRIPTION: This function provides formatted output to an open stream.
 *
 *****************************************************************************/
void ACPI_INTERNAL_VAR_XFACE AcpiOsPrintf (const char *Format, ...)
{
	va_list     ap;

    va_start(ap, Format);
    syslog_valist(LOG_CORE, NULL, 0, Format, ap);
    va_end(ap);
}


/******************************************************************************
 *
 * FUNCTION:     AcpiOsVprintf
 *
 * PARAMETERS:  Format		- A standard print format string.
 *				Args		- Variable printf parameter list.
 *
 * RETURN:      None
 *
 * DESCRIPTION: This function provides formatted output to an open stream via 
 *				the va_list argument format.
 *
 *****************************************************************************/
void AcpiOsVprintf (const char *Format, va_list Args)
{
	syslog_valist(LOG_CORE, NULL, 0, (const utf8_t *)Format, Args);
}


/******************************************************************************
 *
 * FUNCTION:     AcpiOsRedirectOutput
 *
 * PARAMETERS:  Destination		- An open file handle or pointer. Debug output
 *								  will be redirected to this handle/pointer. 
 *								  The format of this parameter is OS-specific. 
 *
 * RETURN:      None
 *
 * DESCRIPTION: This function redirects the output of AcpiOsPrintf and 
 *				AcpiOsVprintf to the specified destination. Usually used to 
 *				redirect output to a file.
 *
 *****************************************************************************/
void AcpiOsRedirectOutput (void *Destination)
{
	return;
}