/*
#include <vortex/common.h>
#include <vortex/acpi/platform/acenv.h>
#include <vortex/acpi/actypes.h>
#include <vortex/acpi/actbl.h>
#include <vortex/acpi/acpiosxf.h>
#include <vortex/acpi/acexcep.h>
*/
#include <vortex/acpi/acpiheaders.h>


/******************************************************************************
 *
 * FUNCTION:    AcpiOsPredefinedOverride
 *
 * PARAMETERS:  PredefinedObject    - A pointer to a predefined object (name 
 *									  and initial value)
 *              NewValue            - Where the new value for the predefined 
 *									  object is returned. NULL if there is no 
 *									  override for this object.
 *
 * RETURN:      AE_OK		Override successful.
 *				Other erros can be added. The actual error codes do not matter 
 *				for anything but debugging.
 *
 * DESCRIPTION: This function allows the host to override the predefined 
 *				objects in the ACPI namespace.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsPredefinedOverride (const ACPI_PREDEFINED_NAMES *InitVal, ACPI_STRING *NewVal)
{
	if (InitVal == NULL || NewVal == NULL) {
		return AE_BAD_PARAMETER;
	}

	*NewVal = (ACPI_STRING)NULL;

	return AE_OK;
}

/******************************************************************************
 *
 * FUNCTION:    AcpiOsTableOverride
 *
 * PARAMETERS:  ExistingTable       - A pointer to the header of the existing 
 *									  ACPI table.
 *              NewTable            - Where the pointer to the replacement 
 *									  table is to be returned. The OSL returns 
 *									  NULL if no replacement is provided.
 *
 * RETURN:      AE_OK		Override successful.
 *				Other erros can be added. The actual error codes do not matter 
 *				for anything but debugging.
 *
 * DESCRIPTION: This function allows the host to override an ACPI table that 
 *				was found in the firmware. The host OS can examine the existing 
 *				table header for the table singature and version number(s) and 
 *				decide to replace it if desired. Note, only the table header is 
 *				guaranteed to be valid and accessible, not the entire table. 
 *				Further, the header is only guaranteed to be valid and 
 *				accessible for the duration of the execution of this function. 
 *				It may be unmapped immediately afterwards.
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsTableOverride (ACPI_TABLE_HEADER *ExistingTable, ACPI_TABLE_HEADER **NewTable)
{
	if (ExistingTable == NULL || NewTable == NULL) {
		*NewTable = (ACPI_TABLE_HEADER *)NULL;
		return  AE_BAD_PARAMETER;
	}
  
	*NewTable = (ACPI_TABLE_HEADER *)NULL;
	
	return  AE_OK;
}


/******************************************************************************
 *
 * FUNCTION:    AcpiOsPhysicalTableOverride
 *
 * PARAMETERS:  ExistingTable       - A pointer to the header of the existing 
 *									  ACPI table.
 *              NewTable            - Where physical address of the replacement 
 *									  table is to be returned. The OSL returns 
 *									  NULL if no replacement is provided.
 *				NewTableLength		- Where the length of the replacement table 
 *									  is returned.
 *
 * RETURN:      AE_OK		Override successful.
 *				Other erros can be added. The actual error codes do not matter 
 *				for anything but debugging.
 *
 * DESCRIPTION: This function allows the host to override an ACPI table that 
 *				was found in the firmware via a physical address and length. 
 *				The host OS can examine the existing table header for the table 
 *				singature and version number(s) and decide to replace it if 
 *				desired. Note, only the table header is guaranteed to be valid 
 *				and accessible, not the entire table. Further, the header is 
 *				only guaranteed to be valid and accessible for the duration of 
 *				the execution of this function. It may be unmapped immediately 
 *				afterwards. When the function exits, ACPICA will create a 
 *				mapping for the new table and manage it for the life of the 
 *				table. Also see AcpiOsTableOverride
 *
 *****************************************************************************/
ACPI_STATUS AcpiOsPhysicalTableOverride (ACPI_TABLE_HEADER *ExistingTable, ACPI_PHYSICAL_ADDRESS *NewAddress, UINT32 *NewTableLength)
{
	*NewAddress = (ACPI_PHYSICAL_ADDRESS)NULL; 
	return AE_SUPPORT;
}