
#include <vortex/acpi/runlengthencoding.h>
#include <vortex/acpi/sharemaster.h>
#include <vortex/acpi/pmcoremon.h>
#include <vortex/acpi/platform/acpi_vortex.h>
#include <vortex/acpi/platform/vortex_acpi_pstates.h>
#include <vortex/perfmon/perfmon.h>
#include <vortex/common.h>
#include <vortex/cpulocal.h>
#include <vortex/rqueue.h>
#include "../../../arch/x86_64/cpu.h"
#include <vortex/acpi/gphtpredictor.h>


#define ENABLE_POWER_MANAGEMENT             

        
//Defines for logging rle encoding stuff (also global sharemaster timeout values)
//#define RLE_ENCODINGS_ENABLE_LOGGING        

//#define SHARE_MASTER_UPDATE_TIME_LOGGING
#define SHARE_MASTER_NUM_SAMPLES            100000
#ifdef SHARE_MASTER_UPDATE_TIME_LOGGING
    uint64      comp_times[SHARE_MASTER_NUM_SAMPLES];
#endif

#define MSR_PLATFORM_INFO                   0x000000ce
#define RLE_BUFFER_LENGTH                   100
typedef struct sequence sequence_t;

share_master_t  *sharemasters[CPU_MAXNUM]; //TODO: PUT SHAREMASTER I CPULOCAL structs ?
objref_t        expert_update_thread_tcbref;
objref_t        smi_detection_thread_tcbref;
objref_t        pm_coremon_rle_trace_queue_ref;

//TODO: REMOVE THIS FUNCTION IF PROVEN INECESSARY
static vxerr_t print_intel_enumeration_of_pstates_information(void)
{
    uint32  arch_perfmon_version, num_perfmon_registers, perfmon_register_width, ebx_vector_length, i, mask;
    uint32  cpuid[4];
    bool_t  enhanced_speedstep_capable;
    uint32  cpuid_leaf_thermal_and_power_management, cpuid_leaf_architectural_performance_monitoring,eax, ebx, ecx, edx;

    //Determine if we have support for MONITOR/MWAIT
    //According to the intel manual we should now check 
    //bit 0 in ecx, which corresponds to index 2 in our 
    //cpuid array which has the following layout:
    // +-----+-----+-----+-----+
    // | eax | ebx | ecx | edx |
    // +-----+-----+-----+-----+

    asm_cpuid(CPUID_LEAF_VERSION, 0, cpuid);
    eax = cpuid[0];
    ebx = cpuid[1];
    ecx = cpuid[2];
    edx = cpuid[3];

    if (ecx & 0x80) {
        enhanced_speedstep_capable = TRUE;
    }

    if (enhanced_speedstep_capable)
        syslog(LOG_DEBUG, "The CPU is Enhanced Intel Speedstep capable");
    else
        syslog(LOG_DEBUG, "The CPU is not EIS capable =(");

    cpuid_leaf_thermal_and_power_management = 0x06;
    asm_cpuid(cpuid_leaf_thermal_and_power_management, 0, cpuid);
    eax = cpuid[0];
    ebx = cpuid[1];
    ecx = cpuid[2];
    edx = cpuid[3];

    if (ecx & 0x01)
        syslog(LOG_DEBUG, "The CPU is P-State hardware coordination feedback capable");
    else
        syslog(LOG_DEBUG, "The CPU is does not support p-state hardware coordination");

    cpuid_leaf_architectural_performance_monitoring = 0x0A;
    asm_cpuid(cpuid_leaf_architectural_performance_monitoring, 0, cpuid);
    eax = cpuid[0];
    ebx = cpuid[1];
    ecx = cpuid[2];
    edx = cpuid[3];

    arch_perfmon_version = (uint32)(0x000000FF & eax);
    if (arch_perfmon_version > 0) {
        syslog(LOG_DEBUG, "Architectural Performance monitoring supported (version %d)", arch_perfmon_version);
        num_perfmon_registers = (uint32)((0x0000FF00 & eax) >> 8);
        perfmon_register_width = (uint32)((0x00FF0000 & eax) >> 16);
        ebx_vector_length     = (uint32)((0xFF000000 & eax) >> 24);
        syslog(LOG_DEBUG, "Num perfmon regs: %d, perfmon reg width: %d, ebx vector length: %d", num_perfmon_registers, perfmon_register_width, ebx_vector_length);

        mask = 0x01;
        for (i = 0; i < 8; i++) {
            if (ebx & mask << i)
                syslog(LOG_DEBUG, "Feature %d is not available", i);
            else 
                syslog(LOG_DEBUG, "Feature %d is available", i);
        }

        if (0x40000000 & ecx)
            syslog(LOG_DEBUG, "Should use Fixed-Function performance counters");
        else
            syslog(LOG_DEBUG, "Should use General-purpose or special-purpose performance counters");
    }   
    else
        syslog(LOG_DEBUG, "Architectural Performance monitoring is not supported");
}



static vxerr_t print_mwait_information(void)
{
    uint32  cpuid_leaf_monitor_mwait, ecx, edx, eax, ebx, tmp, i, cstate;
    uint32  cpuid[4];
    bool_t  interrupts_as_break_even_events;

    //Determine if we have support for MONITOR/MWAIT
    //According to the intel manual we should now check 
    //bit 0 in ecx, which corresponds to index 2 in our 
    //cpuid array which has the following layout:
    // +-----+-----+-----+-----+
    // | eax | ebx | ecx | edx |
    // +-----+-----+-----+-----+

    cpuid_leaf_monitor_mwait = 0x05;
    asm_cpuid(cpuid_leaf_monitor_mwait, 0, cpuid);
    eax = cpuid[0];
    ebx = cpuid[1];
    ecx = cpuid[2];
    edx = cpuid[3];
    if (ecx & 0x01 ) {
        syslog(LOG_DEBUG, "MONITOR MWAIT SUPPORTED!");

        if (ecx & 0x02) {
            interrupts_as_break_even_events = (bool_t)TRUE;
            syslog(LOG_DEBUG, "CPU supports treating interrupts as break-even events, even when interrupts are disabled");
        }

        cstate = 0;
        for (i = 0; i <= 28; i += 4) {
            tmp = edx & (0xF << i);
            tmp = (tmp >> i);
            syslog(LOG_DEBUG, "Number of C%d substates using MWAIT is: %d", cstate, (uint32)tmp);
            cstate++;
        }

        syslog(LOG_DEBUG, "Smallest monitor line (%d bytes)", (uint32)(eax & 0xFFFF));
        syslog(LOG_DEBUG, "Largest monitor line (%d bytes)", (uint32)(ebx & 0xFFFF));
    } else {
        syslog(LOG_CRIT, "MONITOR/MWAIT NOT SUPPORTED");
    }

    return VXERR_OK;
}


#ifdef ENABLE_SELECT_BEST_EXPERT_POLICY
void pm_expert_update_thread(void)
{
    vxerr_t     vxerr;
    rle_t       *rle;
    int64       nrequests;
    uint32      new_expert;
    objref_t    reqreflist[1];
    request_t	*req;

    syslog(LOG_DEBUG, "Expert update thread started at core %d", CPU_ID);
    while (1) {
        
        nrequests = _rq_dequeue(NULL, NULL, &pm_coremon_rle_trace_queue_ref,  1, reqreflist, TIME_NTIME);//<-blockflag
        if (nrequests <= 0) {
            continue;
        } else {

            //Unpack the rle
            req = (request_t *)VxO_REFGETOBJ(&reqreflist[0]);
            closure_args_unpack(req->req_closure, "p", &rle);
            VxO_DECREF(req);

            //Update the weights of the experts according to the rle
            vxerr = update_expert_weights(sharemasters[rle->origin_core], rle);
            if (vxerr != VXERR_OK) {
                syslog(LOG_CRIT, "Something went wrong when attempting to update expert weights...");
            }

            new_expert = calculate_best_expert(sharemasters[rle->origin_core]);
            if (new_expert < 0 || new_expert > sharemasters[rle->origin_core]->num_experts) {
                syslog(LOG_CRIT, "Something went wrong when attempting to calculate new timeout, new_expert = %d", new_expert);
            }
                
            cpulocal[rle->origin_core].cl_cstate_hint = cstate_hints[sharemasters[rle->origin_core]->expert_cstates->elems[new_expert]];
            
            //Now free the memory used by the rle
            destroy_rle(rle);
			
        }
    }
    
}


vxerr_t start_expert_update_thread(void)
{
	vxerr_t     vxerr;
    
    //For verifying stuff while testing
    print_mwait_information();

    //Create the thread
    vxerr = thread_create_fmt(&expert_update_thread_tcbref, 10000, AFFINITY_NAFFINITY, KERNEL_CPUAREF, pm_expert_update_thread, "Expert Update Thread (RLE recipient)");
	if (vxerr != VXERR_OK) {
        syslog(LOG_CRIT, "Unable to create Expert update thread: %s", strvxerr(vxerr));
            goto error;
	}
	    
	//And start it
    vxerr = thread_resume(&expert_update_thread_tcbref, 0);
    if (vxerr != VXERR_OK) {
        syslog(LOG_CRIT, "Unable to start Expert update t thread: %s", strvxerr(vxerr));
        goto error;
	}
    
error:
    return vxerr;
}


vxerr_t init_sharemasters(void)
{
    uint32 i, num_experts;
    vxerr_t vxerr;
    double learning_rate, alpha, c_wasting, c_latency;
  
    /*Share master param setup*/
    learning_rate   = 25.0;
    alpha           = 0.001;
    c_wasting       = 1.0;
    c_latency       = 1.0;
    num_experts     = 2; 

    for (i = 0; i < CPU_DRAFTNUM; i++) {
        
        sharemasters[i] = create_share_master(learning_rate, alpha, num_experts, c_wasting, c_latency);
        if (sharemasters[i] == NULL) {
            vxerr = VXERR_INTERNAL;
            goto error;
        }

        init_expert_weights(sharemasters[i]);

        /*Set the timeout values*/
#ifndef ENABLE_POWER_MANAGEMENT
        syslog(LOG_DEBUG, "Power Management Not Enabled");
#else
        sharemasters[i]->expert_cstates->elems[0] = CSTATE_TYPE_C1;
        sharemasters[i]->expert_cstates->elems[1] = CSTATE_TYPE_C3;
        syslog(LOG_DEBUG,"Created sharemaster for selecting best C-state");
#endif
    }

    vxerr = VXERR_OK;

error:
    return vxerr;
}

#endif 

vxerr_t init_pm_coremon(void)
{
    vxerr_t vxerr;
    uint32 cpu_id;

    vxerr = _rq_new(&pm_coremon_rle_trace_queue_ref, NULL, RESOURCE_METRIC_NONE, RQ_OWNTYPE_ANON, NULL);
	if (vxerr != VXERR_OK) {
		syslog(LOG_CRIT, "_rq_new() returned %s", strvxerr(vxerr));
        goto error;
    }

    //Before we can start any threads, and activate the pm functionality in the idle function
    //We must first allocate all dynamic memory data structures and initialize these
    for (cpu_id = 0; cpu_id < CPU_DRAFTNUM; cpu_id++) {
       
#ifdef ENABLE_SELECT_BEST_EXPERT_POLICY
        cpulocal[cpu_id].cl_core_state_rle = create_rle(RLE_BUFFER_LENGTH, cpu_id);
        if (cpulocal[cpu_id].cl_core_state_rle == NULL) {
            syslog(LOG_CRIT, "Unable to create runlength encoding for core %d", cpu_id);
            vxerr = VXERR_INTERNAL;
            goto error;
        }

        cpulocal[cpu_id].cl_cstates = (cstate_array_t *)getPowerSupportedStates(cpu_id);
        //TOD: Introduce error checking here, this is not robust!

#endif
        cpulocal[cpu_id].cl_acpi_pstate_info = (acpi_pstate_info_t *)acpi_init_acpi_pstate_info(cpu_id);
        if (cpulocal[cpu_id].cl_acpi_pstate_info == NULL) {
            syslog(LOG_CRIT, "Unable to initialize the ACPI pstate information for core %d", cpu_id);
            goto error;
        }


#ifdef ENABLE_PSTATE_CONTROL
        //Start a pstate selector threadl, but create its Global Phase History Table first
        cpulocal[cpu_id].cl_gpht_predictor = gpht_predictor_create(8, 256);
        if (cpulocal[cpu_id].cl_gpht_predictor == NULL) {
            syslog(LOG_CRIT, "Unable to create GPHT predictor on core %d", cpu_id);
            vxerr = VXERR_INTERNAL;
            goto error;
        }
          
        vxerr = start_pstate_selector_thread(cpu_id);
        if (vxerr != VXERR_OK) {
            syslog(LOG_CRIT, "Starting the pstate selector thread for core %d failed", cpu_id);
            goto error;
        }
#endif   

#ifdef ENABLE_POWER_MANAGEMENT
        //Init of dynamic memory data structures complete, 
        //activate pm functionality for the core
        cpulocal[cpu_id].cl_pm_active = (bool_t)TRUE;
#else
        cpulocal[cpu_id].cl_pm_active = (bool_t)FALSE;
#endif

    }

#ifdef ENABLE_SELECT_BEST_EXPERT_POLICY

    vxerr = init_sharemasters();
    if (vxerr != VXERR_OK) {
        syslog(LOG_CRIT, "Initialization of sharemasters failed");
        goto error;
    }

    //Start the C3 timeout update thread
    vxerr = start_expert_update_thread();
    if (vxerr != VXERR_OK) {
        syslog(LOG_CRIT, "Starting of expert update thread failed");
        goto error;
    }

#endif
#ifdef ENABLE_STATIC_TIMEOUT_POLICY
    syslog(LOG_DEBUG, "Starting PowerManagement with static timeout policy (timeout = %d)", STATIC_POLICY_VALUE);
#endif
#ifdef ENABLE_DIRECT_TO_CSTATE_POLICY
    syslog(LOG_DEBUG, "Starting PowerManagement with direct to C-state policy (hint=0x%x)", DIRECT_TO_CSTATE_HINT);
#endif

#endif
error:
    syslog(LOG_DEBUG, "Initialization on pm coremon succeded.");
    return vxerr;
}





